@@ -28,6 +28,7 @@ from typing import List
 from typing import Any
 from typing import Optional
 from typing import Callable
+from typing import TYPE_CHECKING
 # pylint: disable=wrong-import-position
 import sys
 if sys.version_info >= (3, 8):
@@ -35,9 +36,7 @@ if sys.version_info >= (3, 8):
 else:
     from typing_extensions import Literal
 # pylint: enable=wrong-import-position
-InstallStatus = Literal['success', 'cancelled', 'failure']
-OutputCallback = Callable[[str], None]
-CompleteCallback = Callable[[InstallStatus], None]
+import os
 import re
 import signal
 import locale
@@ -46,12 +45,27 @@ from gi import require_version
 # pylint: disable=wrong-import-position
 require_version('GLib', '2.0')
 require_version('Gio', '2.0')
-require_version('Gtk', '3.0')
 from gi.repository import GLib # type: ignore
 from gi.repository import Gio # type: ignore
-from gi.repository import Gtk # type: ignore
 # pylint: enable=wrong-import-position
 from i18n import _, init as i18n_init
+sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'engine'))
+# pylint: disable=import-error, wrong-import-order
+from itb_gtk import Gtk # type: ignore
+if TYPE_CHECKING:
+    # These imports are only for type checkers (mypy). They must not be
+    # executed at runtime because itb_gtk controls the Gtk/Gdk versions.
+    # pylint: disable=reimported
+    from gi.repository import Gtk  # type: ignore
+    # pylint: enable=reimported
+from g_compat_helpers import (
+    add_child,
+    show_all,
+)
+# pylint: enable=import-error, wrong-import-order
+InstallStatus = Literal['success', 'cancelled', 'failure']
+OutputCallback = Callable[[str], None]
+CompleteCallback = Callable[[InstallStatus], None]
 
 LOGGER = logging.getLogger('ibus-typing-booster')
 
@@ -212,7 +226,7 @@ def install_packages_with_dialog(
     vbox = dialog.get_content_area()
     top_label = Gtk.Label(label='')
     top_label.set_xalign(0)
-    vbox.pack_start(top_label, False, False, 6)
+    add_child(vbox, top_label)
     textview = Gtk.TextView()
     textview.set_editable(False)
     textview.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
@@ -220,12 +234,12 @@ def install_packages_with_dialog(
     scrolled = Gtk.ScrolledWindow()
     scrolled.set_vexpand(True)
     scrolled.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-    scrolled.add(textview)
-    vbox.pack_start(scrolled, True, True, 6)
+    add_child(scrolled, textview)
+    add_child(vbox, scrolled)
     progressbar = Gtk.ProgressBar()
     progressbar.set_show_text(True)
     progressbar.set_text('0%')
-    vbox.pack_start(progressbar, False, False, 6)
+    add_child(vbox, progressbar)
     action_area = Gtk.Box()
     action_area.set_orientation(Gtk.Orientation.HORIZONTAL)
     action_area.set_halign(Gtk.Align.END)
@@ -234,18 +248,12 @@ def install_packages_with_dialog(
     action_area.set_hexpand(True)
     action_area.set_vexpand(False)
     action_area.set_spacing(0)
-    vbox.pack_start(action_area, False, False, 6)
-    cancel_button = Gtk.Button()
-    cancel_button_label = Gtk.Label()
-    cancel_button_label.set_text_with_mnemonic(_('_Cancel'))
-    cancel_button.add(cancel_button_label)
-    action_area.pack_end(cancel_button, False, False, 6)
-    close_button = Gtk.Button()
-    close_button_label = Gtk.Label()
-    close_button_label.set_text_with_mnemonic(_('_Close'))
-    close_button.add(close_button_label)
-    action_area.pack_end(close_button, False, False, 6)
-    dialog.show_all()
+    add_child(vbox, action_area)
+    cancel_button = Gtk.Button.new_with_mnemonic(_('_Cancel'))
+    add_child(action_area, cancel_button)
+    close_button = Gtk.Button.new_with_mnemonic(_('_Close'))
+    add_child(action_area, close_button)
+    show_all(dialog)
     close_button.hide()
 
     package_index = 0
@@ -317,13 +325,23 @@ def install_packages_with_dialog(
 
     def on_close(_button: Gtk.Button) -> None:
         '''Close button clicked'''
+        if glib_main_loop is not None and glib_main_loop.is_running():
+            glib_main_loop.quit()
         dialog.destroy()
 
     cancel_button.connect('clicked', on_cancel)
     close_button.connect('clicked', on_close)
     cancel_func = install_packages_sequentially_async(
         packages, on_output=append_line, on_complete=finish)
-    dialog.run()
+    glib_main_loop: Optional[GLib.MainLoop] = None
+    # If a GLib main loop is already running (e.g. from setup UI),
+    # don’t start another one; just return control to it.
+    if GLib.main_depth() > 0: # pylint: disable=no-value-for-parameter
+        # There’s already a running loop
+        return
+    # Otherwise (standalone testing), run our own main loop
+    glib_main_loop = GLib.MainLoop()
+    glib_main_loop.run()
     dialog.destroy()
 
 if __name__ == '__main__':
